#include <mega128.h>
#include <adc.h>
#include <stdlib.h>
#define OCR3A (*(unsigned short *) 0x86)

                                                     //Global variables
unsigned char rx_buffer[RX_BUFFER_SIZE];
unsigned char rx_index;
bit status_rx_buffer=EMPTY; 

unsigned char tx_buffer[TX_BUFFER_SIZE];
unsigned char tx_index;
bit status_tx_buffer=EMPTY;

int *servo_position;


                                   //Interrupt's

interrupt [USART1_RXC] void usart1_rx_isr(void)
{
unsigned char status,data;
status=UCSR1A;
data=UDR1;
if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
   {rx_buffer[rx_index++]=data;}
if (rx_index == RX_BUFFER_SIZE) 
 {
 rx_index=0;
 status_rx_buffer=FULL;  
 }
else
 {status_rx_buffer=EMPTY;}
}


interrupt [USART1_DRE] void usart1_tx_isr(void)
{
  UDR1=tx_buffer[tx_index++];
  if (tx_index == TX_BUFFER_SIZE) 
  {
  tx_index=0; 
  status_tx_buffer=EMPTY;
  TX_INTERRUPT_OFF;
  }
  else
  {status_tx_buffer=FULL;}
}
  

interrupt [TIM3_COMPA] void timer3_compa_isr(void)
{
static char current_servo=1;
static int pause = 18432; 

TCCR3C|=(1<<FOC3A);

        if (current_servo == 9) 
        {
         current_servo = 0;
         OCR3A += pause;
         pause = 18432;
        } 
        else 
        {
         OCR3A += servo_position[current_servo];
         pause -= servo_position[current_servo];
         current_servo++;
        }
}


void main(void)
{



ACSR=0x80;
UCSR1A=0x00;
UCSR1B=(1<<RXCIE1)|(1<<RXEN1)|(1<<TXEN1);
UCSR1C=(1<<UCSZ11)|(1<<UCSZ10);
UBRR1H=0x00;
UBRR1L=0x2f;
#asm("sei")
 
 while (1)
 {
  while(status_rx_buffer==EMPTY);
  switch (FUNCTION)
  {
  case RADAR:
  radar();
  break;

  case ROBOMD2:
  robomd2(PARAM1,PARAM2);
  while(status_tx_buffer==FULL);
  FUNCT_TO_PC=ROBOMD2;
  PARAM1_TO_PC=PARAM1;
  PARAM2_TO_PC=PARAM2;
  return_connect;
  break;

  case SERVO:
  servo(PARAM1,PARAM2);
  while(status_tx_buffer==FULL);
  FUNCT_TO_PC=SERVO;
  PARAM1_TO_PC=PARAM1;
  PARAM2_TO_PC=PARAM2;
  return_connect;
  break;

  case ADC_FUNCT:
  start_adc(PARAM1);
  while(status_tx_buffer==FULL);
  FUNCT_TO_PC=ADC_FUNCT;
  PARAM1_TO_PC=reed_adc();
  PARAM2_TO_PC=0;
  return_connect;
  break;

  default:
  FUNCT_TO_PC=0;
  PARAM1_TO_PC=0;              //
  PARAM2_TO_PC=0;
  return_connect;
  }
 }      
     
}                                              

                                              //      ADC functions

void start_adc (unsigned char adc_chanel)
{
if (adc_chanel>9){adc_chanel=9;}
if (adc_chanel==9)
{ADC_OFF;}
else if
(ADC==OFF)
{ADCSRA=(1<<ADEN)|(1<<ADPS1);}
ADMUX=(1<<REFS0)|(1<ADLAR)|adc_chanel;
ADCSRA|=(1<<ADSC);
}

char reed_adc (void)
{
while ((ADCSRA.4)==0);
ADCSRA|=(1<<ADIF);
return ADCH;
}

                                                  
                                                  
                                                  //Servo functions


void servo (unsigned char servo, unsigned int pos)
{
if ((servo&&pos)==0)
{
servo_off();
return;
}
else if(SERV==OFF){servo_on();}
if (servo>8){servo=8;}
if (servo<1){servo=1;}
if (pos<90){pos=90;}
if (pos>210){pos=210;}
servo_position[servo]=(pos*20);
}
 
void servo_on (void)
{
char i;
servo_position=(int*)malloc(9*sizeof(int));
for (i=0;i<9;i++)
    servo_position[i]=3000;
DDRE|=(1<<DDE3)|(1<<DDE2);
PORTE&=~(1<<PORTE3)|(1<<PORTE2);
PORTE.2=1;
OCR3A=2000;
TCCR3A=(1<<COM3A0);
TCCR3B=(1<<CS31);
ETIMSK|=(1<<OCIE3A);
PORTE.2=0;
}

void servo_off (void)
{
DDRE&=~(1<<DDE3)|(1<<DDE2);
TCCR3A=0x00;
TCCR3B=0x00;
ETIMSK&=~(1<<OCIE3A);
free(servo_position);
}

unsigned int get_servo_position (char servo)
{
unsigned int position;
position=((servo_position[servo]/20));
return position;
}


                                                  //Motor functions



void robomd2 (unsigned char m1, unsigned char m2)
{
 if ((PWM_1==0)&&(PWM_2==0)&&(m1==0)&&(m2==0))
 {
 pwm_off();
 return;
 }
 else
 if(PWM==OFF){pwm_on();}
 motor1(m1);
 motor2(m2);
}

void pwm_on(void)
{
TCCR1A=0x01; 
TCCR1B=0b00001010; 
DDRB|=(1<<DDB6)|(1<<DDB5);
PORTB&=~(1<<PORTB6)|(1<<PORTB7); 
DDRC|=(1<<DDC7)|(1<<DDC6);
PORTC&=~(1<<PORTC7)|(1<<PORTC6); 
}
void pwm_off(void)
{
TCCR1A=0x00; 
TCCR1B=0x00; 
DDRB&=~(1<<DDB6)|(1<<DDB5);
DDRC&=~(1<<DDC7)|(1<<DDC6);
}

void motor1 (unsigned char m1)
{
 if ((m1==0)&&(PWM_1==OFF))
 {
 PWM_OFF(1);
 return;
 }
 else
 if(PWM_1==OFF)
 {PWM_ON(1);}
 
 if(testbit(m1,0))
 {FORWARD_M1;}
 else
 {REVERS_M1;} 
 PWM_1=m1; 

}
 
void motor2 (unsigned char m2)
{
 if ((m2==0)&&(PWM_2==OFF))
 {
 PWM_OFF(2);
 return;
 }
 else
 if(PWM_2==OFF)
 {PWM_ON(2);}
 
 if(testbit(m2,0))
 {FORWARD_M2;}
 else
 {REVERS_M2;} 
 PWM_2=m2;
}


 
   
                                           //radar function
void radar(void)
{
static bit radar_move;
start_adc(SHARP);
servo_on();
switch(radar_move)
{ 
 case 0:
 if(servo_position[RADAR]<=2500)
 {
 radar_move=1;
 break;
 }
 servo_position[RADAR]--;
 break;
 
 case 1:
 if(servo_position[RADAR]>=3500)
 {
 radar_move=0;
 break;
 }
 servo_position[RADAR]++;
 break;
}
while(status_tx_buffer==FULL);
FUNCT_TO_PC=RADAR;
PARAM1_TO_PC=get_servo_position(RADAR);
PARAM2_TO_PC=reed_adc();
return_connect;
}



